@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -20,10 +20,12 @@ sabnzbd.config - Configuration Support
 """
 
 import os
+import re
 import logging
 import threading
+import shutil
 import sabnzbd.misc
-from sabnzbd.constants import CONFIG_VERSION, NORMAL_PRIORITY, DEFAULT_PRIORITY
+from sabnzbd.constants import CONFIG_VERSION, NORMAL_PRIORITY, DEFAULT_PRIORITY, MAX_WIN_DFOLDER
 from sabnzbd.utils import listquote
 from sabnzbd.utils import configobj
 from sabnzbd.decorators import synchronized
@@ -44,19 +46,22 @@ modified = False            # Signals a change in option dictionary
 
 class Option(object):
     """ Basic option class, basic fields """
-    def __init__(self, section, keyword, default_val=None, add=True):
+
+    def __init__(self, section, keyword, default_val=None, add=True, protect=False):
         """ Basic option
             section     : single section or comma-separated list of sections
                           a list will be a hierarchy: "foo, bar" --> [foo][[bar]]
             keyword     : keyword in the (last) section
             default_val : value returned when no value has been set
-            callback    : procedure to call when value is succesfully changed
+            callback    : procedure to call when value is successfully changed
+            protect     : Do not allow setting via the API (specifically set_dict)
         """
         self.__sections = section.split(',')
         self.__keyword = keyword
         self.__default_val = default_val
         self.__value = None
         self.__callback = None
+        self.__protect = protect
 
         # Add myself to the config dictionary
         if add:
@@ -84,10 +89,12 @@ class Option(object):
 
     def get_dict(self, safe=False):
         """ Return value a dictionary """
-        return { self.__keyword : self.get() }
+        return {self.__keyword: self.get()}
 
     def set_dict(self, dict):
         """ Set value based on dictionary """
+        if self.__protect:
+            return False
         try:
             return self.set(dict['value'])
         except KeyError:
@@ -119,11 +126,11 @@ class Option(object):
         return self.__sections, self.__keyword
 
 
-
 class OptionNumber(Option):
     """ Numeric option class, int/float is determined from default value """
-    def __init__(self, section, keyword, default_val=0, minval=None, maxval=None, validation=None, add=True):
-        Option.__init__(self, section, keyword, default_val, add=add)
+
+    def __init__(self, section, keyword, default_val=0, minval=None, maxval=None, validation=None, add=True, protect=False):
+        Option.__init__(self, section, keyword, default_val, add=add, protect=protect)
         self.__minval = minval
         self.__maxval = maxval
         self.__validation = validation
@@ -138,7 +145,7 @@ class OptionNumber(Option):
                 else:
                     value = float(value)
             except ValueError:
-                value = 0
+                value = self._Option__default_val
             if self.__validation:
                 error, val = self.__validation(value)
                 self._Option__set(val)
@@ -153,8 +160,9 @@ class OptionNumber(Option):
 
 class OptionBool(Option):
     """ Boolean option class """
-    def __init__(self, section, keyword, default_val=False, add=True):
-        Option.__init__(self, section, keyword, int(default_val), add=add)
+
+    def __init__(self, section, keyword, default_val=False, add=True, protect=False):
+        Option.__init__(self, section, keyword, int(default_val), add=add, protect=protect)
 
     def set(self, value):
         if value is None:
@@ -168,13 +176,26 @@ class OptionBool(Option):
 
 class OptionDir(Option):
     """ Directory option class """
-    def __init__(self, section, keyword, default_val='', apply_umask=False, create=True, validation=None, add=True):
+
+    def __init__(self, section, keyword, default_val='', apply_umask=False, create=True, validation=None, writable=True, add=True):
         self.__validation = validation
         self.__root = ''   # Base directory for relative paths
         self.__apply_umask = apply_umask
         self.__create = create
+        self.__writable = writable
         Option.__init__(self, section, keyword, default_val, add=add)
 
+    def get(self):
+        """ Return value, corrected for platform """
+        if self._Option__value != None:
+            p = self._Option__value
+        else:
+            p = self._Option__default_val
+        if sabnzbd.WIN32:
+            return p.replace('/', '\\') if '/' in p else p
+        else:
+            return p.replace('\\', '/') if '\\' in p else p
+
     def get_path(self):
         """ Return full absolute path """
         value = self.get()
@@ -182,9 +203,17 @@ class OptionDir(Option):
         if value:
             path = sabnzbd.misc.real_path(self.__root, value)
             if self.__create and not os.path.exists(path):
-                res, path = sabnzbd.misc.create_real_path(self.ident()[1], self.__root, value, self.__apply_umask)
+                res, path = sabnzbd.misc.create_real_path(self.ident()[1], self.__root, value, self.__apply_umask, self.__writable)
         return path
 
+    def test_path(self):
+        """ Return True if path exists """
+        value = self.get()
+        if value:
+            return os.path.exists(sabnzbd.misc.real_path(self.__root, value))
+        else:
+            return False
+
     def set_root(self, root):
         """ Set new root, is assumed to be valid """
         self.__root = root
@@ -202,28 +231,36 @@ class OptionDir(Option):
                 error, value = self.__validation(self.__root, value, self._Option__default_val)
             if not error:
                 if value and (self.__create or create):
-                    res, path = sabnzbd.misc.create_real_path(self.ident()[1], self.__root, value, self.__apply_umask)
+                    res, path = sabnzbd.misc.create_real_path(self.ident()[1], self.__root, value, self.__apply_umask, self.__writable)
                     if not res:
-                        error = Ta("Cannot create %s folder %s") % (self.ident()[1], path)
+                        error = T('Cannot create %s folder %s') % (self.ident()[1], path)
             if not error:
                 self._Option__set(value)
         return error
 
+    def set_create(self, value):
+        """ Set auto-creation value """
+        self.__create = value
+
 
 class OptionList(Option):
     """ List option class """
-    def __init__(self, section, keyword, default_val=None, validation=None, add=True):
+
+    def __init__(self, section, keyword, default_val=None, validation=None, add=True, protect=False):
         self.__validation = validation
         if default_val is None:
             default_val = []
-        Option.__init__(self, section, keyword, default_val, add=add)
+        Option.__init__(self, section, keyword, default_val, add=add, protect=protect)
 
     def set(self, value):
-        """ Set the list given a comma-separated string or a list"""
+        """ Set the list given a comma-separated string or a list """
         error = None
         if value is not None:
             if not isinstance(value, list):
-                value = listquote.simplelist(value)
+                if '"' not in value and ',' not in value:
+                    value = value.split()
+                else:
+                    value = listquote.simplelist(value)
             if self.__validation:
                 error, value = self.__validation(value)
             if not error:
@@ -239,14 +276,16 @@ class OptionList(Option):
         r = len(lst)
         for n in xrange(r):
             txt += lst[n]
-            if n < r-1: txt += ', '
+            if n < r - 1:
+                txt += ', '
         return txt
 
 
 class OptionStr(Option):
     """ String class """
-    def __init__(self, section, keyword, default_val='', validation=None, add=True, strip=True):
-        Option.__init__(self, section, keyword, default_val, add=add)
+
+    def __init__(self, section, keyword, default_val='', validation=None, add=True, strip=True, protect=False):
+        Option.__init__(self, section, keyword, default_val, add=add, protect=protect)
         self.__validation = validation
         self.__strip = strip
 
@@ -261,7 +300,7 @@ class OptionStr(Option):
     def set(self, value):
         """ Set stripped value """
         error = None
-        if type(value) == type('') and self.__strip:
+        if isinstance(value, basestring) and self.__strip:
             value = value.strip()
         if self.__validation:
             error, val = self.__validation(value)
@@ -273,6 +312,7 @@ class OptionStr(Option):
 
 class OptionPassword(Option):
     """ Password class """
+
     def __init__(self, section, keyword, default_val='', add=True):
         Option.__init__(self, section, keyword, default_val, add=add)
         self.get_string = self.get_stars
@@ -292,9 +332,9 @@ class OptionPassword(Option):
     def get_dict(self, safe=False):
         """ Return value a dictionary """
         if safe:
-            return { self._Option__keyword : self.get_stars() }
+            return {self._Option__keyword: self.get_stars()}
         else:
-            return { self._Option__keyword : self.get() }
+            return {self._Option__keyword: self.get()}
 
     def set(self, pw):
         """ Set password, encode it """
@@ -328,41 +368,53 @@ def delete_from_database(section, keyword):
 
 class ConfigServer(object):
     """ Class defining a single server """
+
     def __init__(self, name, values):
 
         self.__name = name
         name = 'servers,' + self.__name
 
+        self.displayname = OptionStr(name, 'displayname', '', add=False)
         self.host = OptionStr(name, 'host', '', add=False)
-        self.port = OptionNumber(name, 'port', 119, 0, 2**16-1, add=False)
+        self.port = OptionNumber(name, 'port', 119, 0, 2 ** 16 - 1, add=False)
         self.timeout = OptionNumber(name, 'timeout', 120, 30, 240, add=False)
         self.username = OptionStr(name, 'username', '', add=False)
         self.password = OptionPassword(name, 'password', '', add=False)
         self.connections = OptionNumber(name, 'connections', 1, 0, 100, add=False)
-        self.fillserver = OptionBool(name, 'fillserver', False, add=False)
         self.ssl = OptionBool(name, 'ssl', False, add=False)
         self.enable = OptionBool(name, 'enable', True, add=False)
         self.optional = OptionBool(name, 'optional', False, add=False)
         self.retention = OptionNumber(name, 'retention', add=False)
+        self.ssl_type = OptionStr(name, 'ssl_type', 't1', add=False)
+        self.send_group = OptionBool(name, 'send_group', False, add=False)
+        self.priority = OptionNumber(name, 'priority', 0, 0, 100, add=False)
+        # 'fillserver' field only here in order to set a proper priority when converting
+        self.fillserver = OptionBool(name, 'fillserver', False, add=False)
+        self.categories = OptionList(name, 'categories', default_val=['Default'], add=False)
+        self.notes = OptionStr(name, 'notes', '', add=False)
 
         self.set_dict(values)
         add_to_database('servers', self.__name, self)
 
     def set_dict(self, values):
         """ Set one or more fields, passed as dictionary """
-        for kw in ('host', 'port', 'timeout', 'username', 'password', 'connections',
-                   'fillserver', 'ssl', 'enable', 'optional', 'retention'):
+        for kw in ('displayname', 'host', 'port', 'timeout', 'username', 'password', 'connections', 'fillserver',
+                   'ssl', 'ssl_type', 'send_group', 'enable', 'optional', 'retention', 'priority',
+                   'categories', 'notes'):
             try:
                 value = values[kw]
             except KeyError:
                 continue
             exec 'self.%s.set(value)' % kw
+            if not self.displayname():
+                self.displayname.set(self.__name)
         return True
 
     def get_dict(self, safe=False):
         """ Return a dictionary with all attributes """
         dict = {}
         dict['name'] = self.__name
+        dict['displayname'] = self.displayname()
         dict['host'] = self.host()
         dict['port'] = self.port()
         dict['timeout'] = self.timeout()
@@ -372,11 +424,15 @@ class ConfigServer(object):
         else:
             dict['password'] = self.password()
         dict['connections'] = self.connections()
-        dict['fillserver'] = self.fillserver()
         dict['ssl'] = self.ssl()
         dict['enable'] = self.enable()
         dict['optional'] = self.optional()
         dict['retention'] = self.retention()
+        dict['ssl_type'] = self.ssl_type()
+        dict['send_group'] = self.send_group()
+        dict['priority'] = self.priority()
+        dict['categories'] = self.categories()
+        dict['notes'] = self.notes()
         return dict
 
     def delete(self):
@@ -384,10 +440,8 @@ class ConfigServer(object):
         delete_from_database('servers', self.__name)
 
     def rename(self, name):
-        """ Give server new identity """
-        delete_from_database('servers', self.__name)
-        self.__name = name
-        add_to_database('servers', self.__name, self)
+        """ Give server new display name """
+        self.displayname.set(name)
 
     def ident(self):
         return 'servers', self.__name
@@ -395,6 +449,7 @@ class ConfigServer(object):
 
 class ConfigCat(object):
     """ Class defining a single category """
+
     def __init__(self, name, values):
         self.__name = name
         name = 'categories,' + name
@@ -436,6 +491,7 @@ class ConfigCat(object):
 
 class OptionFilters(Option):
     """ Filter list class """
+
     def __init__(self, section, keyword, add=True):
         Option.__init__(self, section, keyword, add=add)
         self.set([])
@@ -475,7 +531,7 @@ class OptionFilters(Option):
         dict = {}
         n = 0
         for filter in self.get():
-            dict['filter'+str(n)] = filter
+            dict['filter' + str(n)] = filter
             n = n + 1
         return dict
 
@@ -499,8 +555,10 @@ class OptionFilters(Option):
             self.set(filters)
         return True
 
+
 class ConfigRSS(object):
     """ Class defining a single Feed definition """
+
     def __init__(self, name, values):
         self.__name = name
         name = 'rss,' + name
@@ -552,7 +610,6 @@ class ConfigRSS(object):
         return 'rss', self.__name
 
 
-
 def get_dconfig(section, keyword, nested=False):
     """ Return a config values dictonary,
         Single item or slices based on 'section', 'keyword'
@@ -587,16 +644,15 @@ def get_dconfig(section, keyword, nested=False):
         data = item.get_dict(safe=True)
         if not nested:
             if section in ('servers', 'categories', 'rss'):
-                data = {section : [ data ]}
+                data = {section: [data]}
             else:
-                data = {section : data}
+                data = {section: data}
 
     return True, data
 
 
 def get_config(section, keyword):
-    """ Return a config object, based on 'section', 'keyword'
-    """
+    """ Return a config object, based on 'section', 'keyword' """
     try:
         return database[section][keyword]
     except KeyError:
@@ -605,8 +661,7 @@ def get_config(section, keyword):
 
 
 def set_config(kwargs):
-    """ Set a config item, using values in dictionary
-    """
+    """ Set a config item, using values in dictionary """
     try:
         item = database[kwargs.get('section')][kwargs.get('keyword')]
     except KeyError:
@@ -616,28 +671,41 @@ def set_config(kwargs):
 
 
 def delete(section, keyword):
-    """ Delete specific config item
-    """
+    """ Delete specific config item """
     try:
         database[section][keyword].delete()
     except KeyError:
         return
 
 
-################################################################################
-#
+##############################################################################
 # INI file support
 #
 # This does input and output of configuration to an INI file.
 # It translates this data structure to the config database.
-
+##############################################################################
 @synchronized(SAVE_CONFIG_LOCK)
 def read_config(path):
+    """ Read the complete INI file and check its version number
+        if OK, pass values to config-database
+    """
+    return _read_config(path)
+
+
+def _read_config(path, try_backup=False):
     """ Read the complete INI file and check its version number
         if OK, pass values to config-database
     """
     global CFG, database, modified
 
+    if try_backup or not os.path.exists(path):
+        # Not found, try backup
+        try:
+            shutil.copyfile(path + '.bak', path)
+            try_backup = True
+        except IOError:
+            pass
+
     if not os.path.exists(path):
         # No file found, create default INI file
         try:
@@ -652,23 +720,49 @@ def read_config(path):
             return False, 'Cannot create INI file %s' % path
 
     try:
-        CFG = configobj.ConfigObj(path)
-        try:
-            if int(CFG['__version__']) > int(CONFIG_VERSION):
-                return False, "Incorrect version number %s in %s" % (CFG['__version__'], path)
-        except (KeyError, ValueError):
-            CFG['__version__'] = CONFIG_VERSION
-    except configobj.ConfigObjError, strerror:
-        return False, '"%s" is not a valid configuration file<br>Error message: %s' % (path, strerror)
+        fp = open(path, 'rb')
+        lines = fp.read().split('\n')
+        if len(lines) == 1:
+            fp.seek(0)
+            lines = fp.read().split('\r')
+        lines = [line.rstrip('\r\n') for line in lines]
+        fp.close()
 
-    CFG['__version__'] = CONFIG_VERSION
+        try:
+            # First try UTF-8 encoding
+            CFG = configobj.ConfigObj(lines, default_encoding='utf-8', encoding='utf-8')
+        except UnicodeDecodeError:
+            # Failed, enable retry
+            CFG = {}
+
+        if not re.search(r'utf[ -]*8', CFG.get('__encoding__', ''), re.I):
+            # INI file is still in 8bit ASCII encoding, so try Latin-1 instead
+            CFG = configobj.ConfigObj(lines, default_encoding='cp1252', encoding='cp1252')
+
+    except (IOError, configobj.ConfigObjError, UnicodeEncodeError), strerror:
+        if try_backup:
+            if isinstance(strerror, UnicodeEncodeError):
+                strerror = 'Character encoding of the file is inconsistent'
+            return False, '"%s" is not a valid configuration file<br>Error message: %s' % (path, strerror)
+        else:
+            # Try backup file
+            return _read_config(path, True)
+
+    try:
+        version = sabnzbd.misc.int_conv(CFG['__version__'])
+        if version > int(CONFIG_VERSION):
+            return False, "Incorrect version number %s in %s" % (version, path)
+    except (KeyError, ValueError):
+        pass
+
+    CFG.filename = path
+    CFG.encoding = 'utf-8'
+    CFG['__encoding__'] = u'utf-8'
+    CFG['__version__'] = unicode(CONFIG_VERSION)
 
     if 'misc' in CFG:
         compatibility_fix(CFG['misc'])
 
-    if 'rss' in CFG:
-        newzbin_fix(CFG['rss'])
-
     # Use CFG data to set values for all static options
     for section in database:
         if section not in ('servers', 'categories', 'rss'):
@@ -688,7 +782,6 @@ def read_config(path):
     return True, ""
 
 
-
 @synchronized(SAVE_CONFIG_LOCK)
 def save_config(force=False):
     """ Update Setup file with current option values """
@@ -706,7 +799,7 @@ def save_config(force=False):
                 CFG[section] = {}
             for subsec in database[section]:
                 if section == 'servers':
-                    subsec_mod = subsec.replace('[', '{').replace(']','}')
+                    subsec_mod = subsec.replace('[', '{').replace(']', '}')
                 else:
                     subsec_mod = subsec
                 try:
@@ -731,48 +824,42 @@ def save_config(force=False):
                 else:
                     CFG[sec][kw] = value
 
+    res = False
     filename = CFG.filename
+    bakname = filename + '.bak'
+
+    # Check if file is writable
+    if not sabnzbd.misc.is_writable(filename):
+        logging.error(T('Cannot write to INI file %s'), filename)
+        return res
+
+    # copy current file to backup
     try:
-        # Check if file is writable
-        if not sabnzbd.misc.is_writable(filename):
-            logging.error(Ta('Cannot write to INI file %s'), filename)
-            modified = False
-            return False
+        shutil.copyfile(filename, bakname)
+        shutil.copymode(filename, bakname)
+    except:
+        # Something wrong with the backup,
+        logging.error(T('Cannot create backup file for %s'), bakname)
+        logging.info("Traceback: ", exc_info=True)
+        return res
 
-        # Read current content
-        f = open(filename)
-        data = f.read()
-        f.close()
-
-        tmpname = filename + '.tmp'
-        bakname = filename + '.bak'
-
-        # Write new file
-        f = open(tmpname, 'w')
-        f.write(data)
-        f.close()
-
-        # Update temp file content
-        CFG.filename = tmpname
+    # Write new config file
+    try:
         CFG.write()
-
-        # Rename to backup
-        if os.path.isfile(bakname):
-            os.remove(bakname)
-        os.rename(filename, bakname)
-
-        # Rename temp file, overwriting old one
-        os.rename(tmpname, filename)
-
+        shutil.copymode(bakname, filename)
         modified = False
         res = True
     except:
-        logging.error(Ta('Cannot create backup file for %s'), filename)
-        logging.info("Traceback: ", exc_info = True)
-        res = False
-    CFG.filename = filename
-    return res
+        logging.error(T('Cannot write to INI file %s'), filename)
+        logging.info("Traceback: ", exc_info=True)
+        try:
+            os.remove(filename)
+        except:
+            pass
+        # Restore INI file from backup
+        sabnzbd.misc.renamer(bakname, filename)
 
+    return res
 
 
 def define_servers():
@@ -783,10 +870,15 @@ def define_servers():
     try:
         for server in CFG['servers']:
             svr = CFG['servers'][server]
-            ConfigServer(server.replace('{', '[').replace('}', ']'), svr)
+            s = ConfigServer(server.replace('{', '[').replace('}', ']'), svr)
+            if s.fillserver():
+                # One time conversion of backup to priority 1
+                s.priority.set(1)
+                s.fillserver.set(False)
     except KeyError:
         pass
 
+
 def get_servers():
     global database
     try:
@@ -825,8 +917,8 @@ def get_categories(cat=0):
         database['categories'] = {}
     cats = database['categories']
     if '*' not in cats:
-        ConfigCat('*', {'pp' : old_def('dirscan_opts', '3'), 'script' : old_def('dirscan_script', 'None'), \
-                        'priority' : old_def('dirscan_priority', NORMAL_PRIORITY)})
+        ConfigCat('*', {'pp': old_def('dirscan_opts', '3'), 'script': old_def('dirscan_script', 'None'),
+                        'priority': old_def('dirscan_priority', NORMAL_PRIORITY)})
         save_config(True)
     if not isinstance(cat, int):
         try:
@@ -847,6 +939,7 @@ def define_rss():
     except KeyError:
         pass
 
+
 def get_rss():
     global database
     try:
@@ -854,18 +947,16 @@ def get_rss():
     except KeyError:
         return {}
 
+
 def get_filename():
     global CFG
     return CFG.filename
 
 
-################################################################################
-#
+##############################################################################
 # Default Validation handlers
-#
+##############################################################################
 __PW_PREFIX = '!!!encoded!!!'
-
-#------------------------------------------------------------------------------
 def encode_password(pw):
     """ Encode password in hexadecimal if needed """
     enc = False
@@ -889,9 +980,9 @@ def decode_password(pw, name):
     if pw and pw.startswith(__PW_PREFIX):
         for n in range(len(__PW_PREFIX), len(pw), 2):
             try:
-                ch = chr( int(pw[n] + pw[n+1], 16) )
+                ch = chr(int(pw[n] + pw[n + 1], 16))
             except ValueError:
-                logging.error(Ta('Incorrectly encoded password %s'), name)
+                logging.error(T('Incorrectly encoded password %s'), name)
                 return ''
             decPW += ch
         return decPW
@@ -915,7 +1006,7 @@ def validate_octal(value):
         int(value, 8)
         return None, value
     except:
-        return Ta('%s is not a correct octal value') % value, None
+        return T('%s is not a correct octal value') % value, None
 
 
 def validate_no_unc(root, value, default):
@@ -924,15 +1015,19 @@ def validate_no_unc(root, value, default):
     if value and not value.startswith(r'\\'):
         return validate_notempty(root, value, default)
     else:
-        return Ta('UNC path "%s" not allowed here') % value, None
+        return T('UNC path "%s" not allowed here') % value, None
 
 
 def validate_safedir(root, value, default):
-    """ Allow only when queues are empty and no UNC """
+    """ Allow only when queues are empty and no UNC
+        On Windows path should be small
+    """
+    if sabnzbd.WIN32 and value and len(sabnzbd.misc.real_path(root, value)) >= MAX_WIN_DFOLDER:
+        return T('Error: Path length should be below %s.') % MAX_WIN_DFOLDER, None
     if sabnzbd.empty_queues():
         return validate_no_unc(root, value, default)
     else:
-        return Ta('Error: Queue not empty, cannot change folder.'), None
+        return T('Error: Queue not empty, cannot change folder.'), None
 
 
 def validate_dir_exists(root, value, default):
@@ -941,7 +1036,7 @@ def validate_dir_exists(root, value, default):
     if os.path.exists(p):
         return None, value
     else:
-        return Ta('Folder "%s" does not exist') % p, None
+        return T('Folder "%s" does not exist') % p, None
 
 
 def validate_notempty(root, value, default):
@@ -953,8 +1048,7 @@ def validate_notempty(root, value, default):
 
 
 def create_api_key():
-    """ Return a new randomized API_KEY
-    """
+    """ Return a new randomized API_KEY """
     import time
     try:
         from hashlib import md5
@@ -973,13 +1067,11 @@ def create_api_key():
     return m.hexdigest()
 
 
-#------------------------------------------------------------------------------
-_FIXES = \
-(
-    ('bandwith_limit', 'bandwidth_limit'),
-    ('enable_par_multicore', 'par2_multicore')
+_FIXES = (
+    ('enable_par_multicore', 'par2_multicore'),
 )
 
+
 def compatibility_fix(cf):
     """ Convert obsolete INI entries """
     for item in _FIXES:
@@ -992,10 +1084,3 @@ def compatibility_fix(cf):
                 del cf[old]
             except KeyError:
                 pass
-
-def newzbin_fix(cf):
-    """ Replace old newzbin links """
-    for feed in cf:
-        item = cf[feed].get('uri')
-        if item and 'newzbin.com' in item:
-            cf[feed]['uri'] = item.replace('newzbin.com', 'newzbin2.es')
