@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -31,23 +31,25 @@ from sabnzbd.constants import *
 from sabnzbd.decorators import synchronized
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
-from sabnzbd.misc import cat_convert, sanitize_foldername, wildcard_to_re, cat_to_opts
+from sabnzbd.misc import cat_convert, sanitize_foldername, wildcard_to_re, cat_to_opts, \
+    match_str, from_units, int_conv, get_urlbase
 import sabnzbd.emailer as emailer
-from sabnzbd.encoding import latin1, unicoder, xml_name
+from sabnzbd.encoding import unicoder, xml_name
 
 import sabnzbd.utils.feedparser as feedparser
 
 __RSS = None  # Global pointer to RSS-scanner instance
 
 
-################################################################################
-# Wrapper functions                                                            #
-################################################################################
+##############################################################################
+# Wrapper functions
+##############################################################################
 
 def init():
     global __RSS
     __RSS = RSSQueue()
 
+
 def stop():
     global __RSS
     if __RSS:
@@ -57,25 +59,36 @@ def stop():
         except:
             pass
 
+
 def del_feed(feed):
     global __RSS
-    if __RSS: __RSS.delete(feed)
+    if __RSS:
+        __RSS.delete(feed)
+
 
 def run_feed(feed, download, ignoreFirst=False, force=False, readout=True):
     global __RSS
-    if __RSS: return __RSS.run_feed(feed, download, ignoreFirst, force=force, readout=readout)
+    if __RSS:
+        return __RSS.run_feed(feed, download, ignoreFirst, force=force, readout=readout)
+
 
 def show_result(feed):
     global __RSS
-    if __RSS: return __RSS.show_result(feed)
+    if __RSS:
+        return __RSS.show_result(feed)
 
-def flag_downloaded(feed, id):
-    global __RSS
-    if __RSS: __RSS.flag_downloaded(feed, id)
 
-def lookup_url(feed, id):
+def flag_downloaded(feed, fid):
     global __RSS
-    if __RSS: return __RSS.lookup_url(feed, id)
+    if __RSS:
+        __RSS.flag_downloaded(feed, fid)
+
+
+def lookup_url(feed, fid):
+    global __RSS
+    if __RSS:
+        return __RSS.lookup_url(feed, fid)
+
 
 def run_method():
     global __RSS
@@ -84,24 +97,40 @@ def run_method():
     else:
         return None
 
+
+def next_run(t=None):
+    global __RSS
+    if __RSS:
+        if t:
+            __RSS.next_run = t
+        else:
+            return __RSS.next_run
+    else:
+        return time.time()
+
+
 def save():
     global __RSS
-    if __RSS: __RSS.save()
+    if __RSS:
+        __RSS.save()
+
 
 def clear_feed(feed):
     global __RSS
-    if __RSS: __RSS.clear_feed(feed)
+    if __RSS:
+        __RSS.clear_feed(feed)
+
 
 def clear_downloaded(feed):
     global __RSS
-    if __RSS: __RSS.clear_downloaded(feed)
+    if __RSS:
+        __RSS.clear_downloaded(feed)
 
 
-################################################################################
+##############################################################################
 
 def notdefault(item):
-    """ Return True if not 'Default'/'*'/''
-    """
+    """ Return True if not 'Default|''|*' """
     return bool(item) and str(item).lower() not in ('default', '*', '', str(DEFAULT_PRIORITY))
 
 
@@ -112,6 +141,7 @@ def ListUris():
         uris.append(uri)
     return uris
 
+
 def convert_filter(text):
     """ Return compiled regex.
         If string starts with re: it's a real regex
@@ -125,17 +155,17 @@ def convert_filter(text):
     try:
         return re.compile(txt, re.I)
     except:
-        logging.error(Ta('Could not compile regex: %s'), text)
+        logging.debug('Could not compile regex: %s', text)
         return None
 
-_EXPIRE_SEC = 3*24*3600 # 3 days
+
 def remove_obsolete(jobs, new_jobs):
     """ Expire G/B links that are not in new_jobs (mark them 'X')
         Expired links older than 3 days are removed from 'jobs'
     """
     now = time.time()
-    limit =  now - _EXPIRE_SEC
-    olds  = jobs.keys()
+    limit = now - 259200  # 3days (3x24x3600)
+    olds = jobs.keys()
     for old in olds:
         tm = jobs[old]['time']
         if old not in new_jobs:
@@ -147,10 +177,14 @@ def remove_obsolete(jobs, new_jobs):
 
 
 LOCK = threading.RLock()
+
+
 class RSSQueue(object):
+
     def __init__(self):
         def check_str(p):
-            return p is None or p == '' or isinstance(p, str)
+            return p is None or p == '' or isinstance(p, basestring)
+
         def check_int(p):
             try:
                 int(p)
@@ -159,6 +193,8 @@ class RSSQueue(object):
                 return False
 
         self.jobs = {}
+        self.next_run = time.time()
+
         try:
             defined = config.get_rss().keys()
             feeds = sabnzbd.load_admin(RSS_FILE_NAME)
@@ -194,16 +230,26 @@ class RSSQueue(object):
                                     raise IndexError
                                 if item.get('status', ' ')[0] not in ('D', 'G', 'B', 'X'):
                                     item['status'] = 'X'
-                                if not isinstance(item.get('url'), unicode): item['url'] = ''
-                                item['url'] = item['url'].replace('www.newzbin.com', cfg.newzbin_url())
-                                if not check_str(item.get('cat')): item['cat'] = ''
-                                if not check_str(item.get('orgcat')): item['orgcat'] = ''
-                                if not check_str(item.get('pp')): item['pp'] = '3'
-                                if not check_str(item.get('script')): item['script'] = 'None'
-                                if not check_str(item.get('prio')): item['prio'] = '-100'
-                                if not check_int(item.get('rule', 0)): item['rule'] = 0
-                                if not isinstance(item.get('time'), float): item['time'] = time.time()
-                                if not check_int(item.get('order', 0)): item.get['order'] = 0
+                                if not isinstance(item.get('url'), unicode):
+                                    item['url'] = ''
+                                if not check_str(item.get('cat')):
+                                    item['cat'] = ''
+                                if not check_str(item.get('orgcat')):
+                                    item['orgcat'] = ''
+                                if not check_str(item.get('pp')):
+                                    item['pp'] = '3'
+                                if not check_str(item.get('script')):
+                                    item['script'] = 'None'
+                                if not check_str(item.get('prio')):
+                                    item['prio'] = '-100'
+                                if not check_int(item.get('rule', 0)):
+                                    item['rule'] = 0
+                                if not check_int(item.get('size', 0L)):
+                                    item['size'] = 0L
+                                if not isinstance(item.get('time'), float):
+                                    item['time'] = time.time()
+                                if not check_int(item.get('order', 0)):
+                                    item.get['order'] = 0
                                 self.jobs[feed][link] = item
                             except (KeyError, IndexError):
                                 logging.info('Incorrect entry in %s detected, discarding %s', RSS_FILE_NAME, item)
@@ -220,7 +266,7 @@ class RSSQueue(object):
         #               '*' added means: from the initial batch
         #               '-' added to 'D' means downloaded, but not displayed anymore
         #           title : Title
-        #           url : URL or MsgId
+        #           url : URL
         #           cat : category
         #           orgcat : category as read from feed
         #           pp : pp
@@ -228,6 +274,7 @@ class RSSQueue(object):
         #           prio : priority
         #           time : timestamp (used for time-based clean-up)
         #           order : order in the RSS feed
+        #           size : size in bytes
 
         self.shutdown = False
 
@@ -252,7 +299,6 @@ class RSSQueue(object):
                         return fd
             return ''
 
-
         if not feed:
             return 'No such feed'
 
@@ -263,13 +309,14 @@ class RSSQueue(object):
         try:
             feeds = config.get_rss()[feed]
         except KeyError:
-            logging.error(Ta('Incorrect RSS feed description "%s"'), feed)
-            logging.info("Traceback: ", exc_info = True)
+            logging.error(T('Incorrect RSS feed description "%s"'), feed)
+            logging.info("Traceback: ", exc_info=True)
             return T('Incorrect RSS feed description "%s"') % feed
 
         uri = feeds.uri()
         defCat = feeds.cat()
-        if not notdefault(defCat):
+        import sabnzbd.api
+        if not notdefault(defCat) or defCat not in sabnzbd.api.list_cats(default=False):
             defCat = None
         defPP = feeds.pp()
         if not notdefault(defPP):
@@ -297,19 +344,16 @@ class RSSQueue(object):
             rePPs.append(filter[1])
             reScripts.append(filter[2])
             reTypes.append(filter[3])
-            regexes.append(convert_filter(filter[4]))
+            if filter[3] in ('<', '>', 'F'):
+                regexes.append(filter[4])
+            else:
+                regexes.append(convert_filter(filter[4]))
             rePrios.append(filter[5])
             reEnabled.append(filter[6] != '0')
         regcount = len(regexes)
 
         # Set first if this is the very first scan of this URI
-        if feed not in self.jobs:
-            self.jobs[feed] = {}
-        first = not bool(self.jobs[feed])
-
-        jobs = self.jobs[feed]
-
-        first = first and ignoreFirst
+        first = (feed not in self.jobs) and ignoreFirst
 
         # Add sabnzbd's custom User Agent
         feedparser.USER_AGENT = 'SABnzbd+/%s' % sabnzbd.version.__version__
@@ -319,67 +363,81 @@ class RSSQueue(object):
             uri += '&dl=1'
 
         # Read the RSS feed
+        msg = None
+        entries = None
         if readout:
             uri = uri.replace(' ', '%20')
             logging.debug("Running feedparser on %s", uri)
             d = feedparser.parse(uri.replace('feed://', 'http://'))
             logging.debug("Done parsing %s", uri)
             if not d:
-                msg = Ta('Failed to retrieve RSS from %s: %s') % (uri, '?')
+                msg = T('Failed to retrieve RSS from %s: %s') % (uri, '?')
                 logging.info(msg)
                 return unicoder(msg)
 
             status = d.get('status', 999)
             if status in (401, 402, 403):
-                msg = Ta('Do not have valid authentication for feed %s') % feed
+                msg = T('Do not have valid authentication for feed %s') % feed
+                logging.info(msg)
+                return unicoder(msg)
+            if status >= 500 and status <= 599:
+                msg = T('Server side error (server code %s); could not get %s on %s') % (status, feed, uri)
                 logging.info(msg)
                 return unicoder(msg)
 
             entries = d.get('entries')
             if 'bozo_exception' in d and not entries:
-                msg = Ta('Failed to retrieve RSS from %s: %s') % (uri, xml_name(str(d['bozo_exception'])))
+                msg = str(d['bozo_exception'])
+                if 'CERTIFICATE_VERIFY_FAILED' in msg:
+                    msg = T('Server %s uses an untrusted HTTPS certificate') % get_urlbase(uri)
+                    logging.error(msg)
+                else:
+                    msg = T('Failed to retrieve RSS from %s: %s') % (uri, xml_name(msg))
                 logging.info(msg)
                 return unicoder(msg)
             if not entries:
-                msg = Ta('RSS Feed %s was empty') % uri
+                msg = T('RSS Feed %s was empty') % uri
                 logging.info(msg)
+
+        if feed not in self.jobs:
+            self.jobs[feed] = {}
+        jobs = self.jobs[feed]
+        if readout:
+            if not entries:
                 return unicoder(msg)
         else:
             entries = jobs.keys()
+            # Sort in the order the jobs came from the feed
+            entries.sort(lambda x, y: jobs[x].get('order', 0) - jobs[y].get('order', 0))
 
         order = 0
         # Filter out valid new links
         for entry in entries:
-            if self.shutdown: return
+            if self.shutdown:
+                return
 
             if readout:
                 try:
-                    link, category = _get_link(uri, entry)
+                    link, category, size = _get_link(uri, entry)
                 except (AttributeError, IndexError):
                     link = None
-                    category = ''
-                    logging.info(Ta('Incompatible feed') + ' ' + uri)
-                    logging.info("Traceback: ", exc_info = True)
+                    category = u''
+                    size = 0L
+                    logging.info(T('Incompatible feed') + ' ' + uri)
+                    logging.info("Traceback: ", exc_info=True)
                     return T('Incompatible feed')
-                category = latin1(category)
-                # Make sure only latin-1 encodable characters occur
-                atitle = latin1(entry.title)
-                title = unicoder(atitle)
+                title = entry.title
             else:
                 link = entry
                 category = jobs[link].get('orgcat', '')
                 if category in ('', '*'):
                     category = None
-                atitle = latin1(jobs[link].get('title', ''))
-                title = unicoder(atitle)
+                title = jobs[link].get('title', '')
+                size = jobs[link].get('size', 0L)
 
             if link:
                 # Make sure spaces are quoted in the URL
-                if 'nzbclub.com' in link:
-                    link, path = os.path.split(link.strip())
-                    link = '%s/%s' % (link, urllib.quote(path))
-                else:
-                    link = link.strip().replace(' ','%20')
+                link = link.strip().replace(' ', '%20')
 
                 newlinks.append(link)
 
@@ -389,15 +447,22 @@ class RSSQueue(object):
                     jobstat = 'N'
                 if jobstat in 'NGB' or (jobstat == 'X' and readout):
                     # Match this title against all filters
-                    logging.debug('Trying title %s', atitle)
+                    logging.debug('Trying title %s', title)
                     result = False
                     myCat = defCat
                     myPP = defPP
                     myScript = defScript
                     myPrio = defPrio
                     n = 0
+                    if 'F' in reTypes:
+                        season, episode = sabnzbd.newsunpack.analyse_show(title)[1:3]
+                        season = int_conv(season)
+                        episode = int_conv(episode)
+                    else:
+                        season = episode = 0
 
-                    # Match against all filters until an postive or negative match
+                    # Match against all filters until an positive or negative match
+                    logging.debug('Size %s for %s', size, title)
                     for n in xrange(regcount):
                         if reEnabled[n]:
                             if category and reTypes[n] == 'C':
@@ -406,8 +471,26 @@ class RSSQueue(object):
                                     logging.debug("Filter rejected on rule %d", n)
                                     result = False
                                     break
+                            elif reTypes[n] == '<' and size and from_units(regexes[n]) < size:
+                                # "Size at most" : too large
+                                logging.debug('Filter rejected on rule %d', n)
+                                result = False
+                                break
+                            elif reTypes[n] == '>' and size and from_units(regexes[n]) > size:
+                                # "Size at least" : too small
+                                logging.debug('Filter rejected on rule %d', n)
+                                result = False
+                                break
+                            elif reTypes[n] == 'F' and not ep_match(season, episode, regexes[n]):
+                                # "Starting from SxxEyy", too early episode
+                                logging.debug('Filter requirement match on rule %d', n)
+                                result = False
+                                break
                             else:
-                                found = re.search(regexes[n], title)
+                                if regexes[n]:
+                                    found = re.search(regexes[n], title)
+                                else:
+                                    found = False
                                 if reTypes[n] == 'M' and not found:
                                     logging.debug("Filter rejected on rule %d", n)
                                     result = False
@@ -445,7 +528,7 @@ class RSSQueue(object):
 
                     if cfg.no_dupes() and dup_title(title):
                         if cfg.no_dupes() == 1:
-                            logging.info("Ignoring duplicate job %s", atitle)
+                            logging.info("Ignoring duplicate job %s", title)
                             continue
                         else:
                             myPrio = DUP_PRIORITY
@@ -458,12 +541,12 @@ class RSSQueue(object):
                     else:
                         star = first
                     if result:
-                        _HandleLink(jobs, link, title, 'G', category, myCat, myPP, myScript,
+                        _HandleLink(jobs, link, title, size, 'G', category, myCat, myPP, myScript,
                                     act, star, order, priority=myPrio, rule=str(n))
                         if act:
                             new_downloads.append(title)
                     else:
-                        _HandleLink(jobs, link, title, 'B', category, myCat, myPP, myScript,
+                        _HandleLink(jobs, link, title, size, 'B', category, myCat, myPP, myScript,
                                     False, star, order, priority=myPrio, rule=str(n))
             order += 1
 
@@ -474,22 +557,21 @@ class RSSQueue(object):
         remove_obsolete(jobs, newlinks)
         return ''
 
-
     def run(self):
-        """ Run all the URI's and filters
-        """
+        """ Run all the URI's and filters """
         if not sabnzbd.PAUSED_ALL:
             active = False
+            if self.next_run < time.time():
+                self.next_run = time.time() + cfg.rss_rate.get() * 60
             feeds = config.get_rss()
             for feed in feeds.keys():
                 try:
                     if feeds[feed].enable.get():
-                        if not active:
-                            logging.info('Starting scheduled RSS read-out')
+                        logging.info('Starting scheduled RSS read-out for "%s"', feed)
                         active = True
                         self.run_feed(feed, download=True, ignoreFirst=True)
                         # Wait 15 seconds, else sites may get irritated
-                        for x in xrange(15):
+                        for unused in xrange(15):
                             if self.shutdown:
                                 return
                             else:
@@ -499,8 +581,7 @@ class RSSQueue(object):
                     pass
             if active:
                 self.save()
-                logging.info('Finished scheduled RSS read-out')
-
+                logging.info('Finished scheduled RSS read-outs')
 
     @synchronized(LOCK)
     def show_result(self, feed):
@@ -522,11 +603,11 @@ class RSSQueue(object):
             del self.jobs[feed]
 
     @synchronized(LOCK)
-    def flag_downloaded(self, feed, id):
+    def flag_downloaded(self, feed, fid):
         if feed in self.jobs:
             lst = self.jobs[feed]
             for link in lst:
-                if lst[link].get('url', '') == id:
+                if lst[link].get('url', '') == fid:
                     lst[link]['status'] = 'D'
 
     @synchronized(LOCK)
@@ -553,82 +634,74 @@ class RSSQueue(object):
                     self.jobs[feed][item]['status'] = 'D-'
 
 
-RE_NEWZBIN = re.compile(r'(newz)(bin|xxx|bin2)\.[\w]+/browse/post/(\d+)', re.I)
-
-def _HandleLink(jobs, link, title, flag, orgcat, cat, pp, script, download, star, order,
+def _HandleLink(jobs, link, title, size, flag, orgcat, cat, pp, script, download, star, order,
                 priority=NORMAL_PRIORITY, rule=0):
     """ Process one link """
-    if script == '': script = None
-    if pp == '': pp = None
+    if script == '':
+        script = None
+    if pp == '':
+        pp = None
 
     jobs[link] = {}
     jobs[link]['order'] = order
     jobs[link]['orgcat'] = orgcat
+    jobs[link]['size'] = size
     if special_rss_site(link):
         nzbname = None
     else:
-        nzbname = sanitize_foldername(title)
-    m = RE_NEWZBIN.search(link)
-    if m and m.group(1).lower() == 'newz' and m.group(2) and m.group(3):
-        if download:
-            jobs[link]['status'] = 'D'
-            jobs[link]['title'] = title
-            logging.info("Adding %s (%s) to queue", m.group(3), title)
-            sabnzbd.add_msgid(m.group(3), pp=pp, script=script, cat=cat, priority=priority, nzbname=nzbname)
-        else:
-            if star:
-                jobs[link]['status'] = flag + '*'
-            else:
-                jobs[link]['status'] = flag
-            jobs[link]['title'] = title
-            jobs[link]['url'] = m.group(3)
-            jobs[link]['cat'] = cat
-            jobs[link]['pp'] = pp
-            jobs[link]['script'] = script
-            jobs[link]['prio'] = str(priority)
+        nzbname = title
+
+    if download:
+        jobs[link]['status'] = 'D'
+        jobs[link]['title'] = title
+        logging.info("Adding %s (%s) to queue", link, title)
+        sabnzbd.add_url(link, pp=pp, script=script, cat=cat, priority=priority, nzbname=nzbname)
     else:
-        if download:
-            jobs[link]['status'] = 'D'
-            jobs[link]['title'] = title
-            logging.info("Adding %s (%s) to queue", link, title)
-            sabnzbd.add_url(link, pp=pp, script=script, cat=cat, priority=priority, nzbname=nzbname)
+        if star:
+            jobs[link]['status'] = flag + '*'
         else:
-            if star:
-                jobs[link]['status'] = flag + '*'
-            else:
-                jobs[link]['status'] = flag
-            jobs[link]['title'] = title
-            jobs[link]['url'] = link
-            jobs[link]['cat'] = cat
-            jobs[link]['pp'] = pp
-            jobs[link]['script'] = script
-            jobs[link]['prio'] = str(priority)
+            jobs[link]['status'] = flag
+        jobs[link]['title'] = title
+        jobs[link]['url'] = link
+        jobs[link]['cat'] = cat
+        jobs[link]['pp'] = pp
+        jobs[link]['script'] = script
+        jobs[link]['prio'] = str(priority)
 
     jobs[link]['time'] = time.time()
     jobs[link]['rule'] = rule
 
+
 def _get_link(uri, entry):
     """ Retrieve the post link from this entry
-        Returns (link, category)
+        Returns (link, category, size)
     """
-    link = None
+    link = None  # @UnusedVariable -- pep8 bug?
     category = ''
+    size = 0L
     uri = uri.lower()
-    if 'newzbin.' in uri or 'newzxxx.'in uri or 'newzbin2.' in uri:
-        link = entry.link
-        if not (link and '/post/' in link.lower()):
-            # Use alternative link
-            link = entry.links[0].href
-    else:
-        # Try standard link first
-        link = entry.link
-        if not link:
-            link = entry.links[0].href
-        if encl_sites(uri, link):
-            try:
-                link = entry.enclosures[0]['href']
-            except:
-                pass
+
+    _RE_SIZE1 = re.compile(r'Size:\s*(\d+\.\d+\s*[KMG]{0,1})B\W*', re.I)
+    _RE_SIZE2 = re.compile(r'\W*(\d+\.\d+\s*[KMG]{0,1})B\W*', re.I)
+    # Try to find size in Description
+    try:
+        desc = entry.description.replace('\n', ' ')
+        m = _RE_SIZE1.search(desc) or _RE_SIZE2.search(desc)
+        if m:
+            size = from_units(m.group(1))
+            logging.debug('Found size %s for %s', size, uri)
+    except:
+        pass
+
+    # Try standard link first
+    link = entry.link
+    if not link:
+        link = entry.links[0].href
+    if encl_sites(uri, link):
+        try:
+            link = entry.enclosures[0]['href']
+        except:
+            pass
 
     if link and 'http' in link.lower():
         try:
@@ -637,30 +710,40 @@ def _get_link(uri, entry):
             try:
                 category = entry.category
             except:
-                try: # nzb.su
+                try:  # nzb.su
                     category = entry.tags[0]['term']
                 except:
-                    try: # nzbmatrix.com
+                    try:
                         category = entry.description
                     except:
                         category = ''
-        return link, category
+        return link, category, size
     else:
-        logging.warning(Ta('Empty RSS entry found (%s)'), link)
-        return None, ''
+        logging.warning(T('Empty RSS entry found (%s)'), link)
+        return None, '', 0L
 
 
 def special_rss_site(url):
-    """ Return True if url describes an RSS site with odd titles
-    """
-    return cfg.rss_filenames() or 'nzbindex.nl/' in url or 'nzbindex.com/' in url or 'nzbclub.com/' in url
+    """ Return True if url describes an RSS site with odd titles """
+    return cfg.rss_filenames() or match_str(url, cfg.rss_odd_titles())
 
 
-_ENCL_SITES = ('nzbindex.nl', 'nzbindex.com', 'animeusenet.org', 'nzbclub.com')
 def encl_sites(url, link):
-    """ Return True if url or link match sites that use enclosures
-    """
+    """ Return True if url or link matches sites that use enclosures """
+    _ENCL_SITES = ('nzbindex.nl', 'nzbindex.com', 'animeusenet.org', 'nzbclub.com')
     for site in _ENCL_SITES:
         if site in url or (link and site in link):
             return True
     return False
+
+
+def ep_match(season, episode, expr):
+    """ Return True if season, episode is at or above expected """
+    _RE_SP = re.compile(r's*(\d+)[ex](\d+)', re.I)
+    m = _RE_SP.search(expr)
+    if m:
+        req_season = int(m.group(1))
+        req_episode = int(m.group(2))
+        return season > req_season or (season == req_season and episode >= req_episode)
+    else:
+        return True
